@@ -2,6 +2,8 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE RankNTypes #-}
 -- | Some helpers for parsing data out of a raw WAI 'Request'.
 
 module Network.Wai.Parse
@@ -50,6 +52,10 @@ import Data.Either (partitionEithers)
 import Control.Monad (when, unless)
 import Control.Monad.Trans.Class (lift)
 import Control.Monad.Trans.Resource (allocate, release, register)
+#if MIN_VERSION_conduit(1, 0, 0)
+import Data.Conduit.Internal (Pipe (NeedInput, HaveOutput), (>+>), withUpstream, Sink (..), injectLeftovers, ConduitM (..))
+import Data.Void (Void)
+#endif
 
 breakDiscard :: Word8 -> S.ByteString -> (S.ByteString, S.ByteString)
 breakDiscard w s =
@@ -184,7 +190,11 @@ conduitRequestBody _ UrlEncoded = do
 conduitRequestBody backend (Multipart bound) =
     parsePieces backend $ S8.pack "--" `S.append` bound
 
+#if MIN_VERSION_conduit(1, 0, 0)
+takeLine :: Monad m => Consumer S.ByteString m (Maybe S.ByteString)
+#else
 takeLine :: Monad m => Pipe S.ByteString S.ByteString o u m (Maybe S.ByteString)
+#endif
 takeLine =
     go id
   where
@@ -199,7 +209,11 @@ takeLine =
                     when (S.length y > 1) $ leftover $ S.drop 1 y
                     return $ Just $ killCR x
 
+#if MIN_VERSION_conduit(1, 0, 0)
+takeLines :: Consumer S.ByteString (ResourceT IO) [S.ByteString]
+#else
 takeLines :: Pipe S.ByteString S.ByteString o u (ResourceT IO) [S.ByteString]
+#endif
 takeLines = do
     res <- takeLine
     case res of
@@ -212,7 +226,11 @@ takeLines = do
 
 parsePieces :: BackEnd y
             -> S.ByteString
+#if MIN_VERSION_conduit(1, 0, 0)
+            -> ConduitM S.ByteString (Either Param (File y)) (ResourceT IO) ()
+#else
             -> Pipe S.ByteString S.ByteString (Either Param (File y)) u (ResourceT IO) ()
+#endif
 parsePieces sink bound =
     loop
   where
@@ -287,13 +305,41 @@ sinkTillBound' :: S.ByteString
                -> S.ByteString
                -> FileInfo ()
                -> BackEnd y
+#if MIN_VERSION_conduit(1, 0, 0)
+               -> ConduitM S.ByteString o (ResourceT IO) (Bool, y)
+#else
                -> Pipe S.ByteString S.ByteString o u (ResourceT IO) (Bool, y)
-sinkTillBound' bound name fi sink = conduitTillBound bound >+> withUpstream (sinkToPipe $ sink name fi)
+#endif
+sinkTillBound' bound name fi sink =
+#if MIN_VERSION_conduit(1, 0, 0)
+    ConduitM $ anyOutput $
+#endif
+    conduitTillBound bound >+> withUpstream (fix $ sink name fi)
+  where
+#if MIN_VERSION_conduit(1, 0, 0)
+    fix :: Sink S8.ByteString (ResourceT IO) y -> Pipe Void S8.ByteString Void Bool (ResourceT IO) y
+    fix (ConduitM p) = ignoreTerm >+> injectLeftovers p
+    ignoreTerm = await' >>= maybe (return ()) (\x -> yield' x >> ignoreTerm)
+    await' = NeedInput (return . Just) (const $ return Nothing)
+    yield' = HaveOutput (return ()) (return ())
+
+    anyOutput p = p >+> dropInput
+    dropInput = NeedInput (const dropInput) return
+#else
+    fix = sinkToPipe
+#endif
 
 conduitTillBound :: Monad m
                  => S.ByteString -- bound
+#if MIN_VERSION_conduit(1, 0, 0)
+                 -> Pipe S.ByteString S.ByteString S.ByteString () m Bool
+#else
                  -> Pipe S.ByteString S.ByteString S.ByteString u m Bool
+#endif
 conduitTillBound bound =
+#if MIN_VERSION_conduit(1, 0, 0)
+    unConduitM $
+#endif
     go id
   where
     go front = await >>= maybe (close front) (push front)
@@ -323,11 +369,26 @@ conduitTillBound bound =
 sinkTillBound :: S.ByteString
               -> (x -> S.ByteString -> IO x)
               -> x
+#if MIN_VERSION_conduit(1, 0, 0)
+              -> Consumer S.ByteString (ResourceT IO) (Bool, x)
+#else
               -> Pipe S.ByteString S.ByteString o u (ResourceT IO) (Bool, x)
+#endif
 sinkTillBound bound iter seed0 =
-    conduitTillBound bound >+> withUpstream (CL.foldM iter' seed0)
+#if MIN_VERSION_conduit(1, 0, 0)
+    ConduitM $
+#endif
+    (conduitTillBound bound >+> (withUpstream $ ij $ CL.foldM iter' seed0))
   where
     iter' a b = liftIO $ iter a b
+#if MIN_VERSION_conduit(1, 0, 0)
+    ij (ConduitM p) = ignoreTerm >+> injectLeftovers p
+    ignoreTerm = await' >>= maybe (return ()) (\x -> yield' x >> ignoreTerm)
+    await' = NeedInput (return . Just) (const $ return Nothing)
+    yield' = HaveOutput (return ()) (return ())
+#else
+    ij = id
+#endif
 
 parseAttrs :: S.ByteString -> [(S.ByteString, S.ByteString)]
 parseAttrs = map go . S.split 59 -- semicolon
