@@ -2,4 +2,4 @@ import Test.Hspec.Monadic
 import qualified WaiExtraTest
 
 main :: IO ()
-main = hspecX WaiExtraTest.specs
+main = hspec WaiExtraTest.specs
